/**************************************************************************

Copyright 1996 Richard Mairesse

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appears in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation. No responsibility is assumed for the use
or reliability of this software.

Author:
  Richard Mairesse                                Tel: 0032 81 725009
  Facultes Universitaires Notre-Dame de la Paix   Fax: 0032 81 725023
  Centre de Calcul                                Richard.Mairesse@fundp.ac.be
  rue Grandgagnage 21
  5000 Namur
  Belgium

**************************************************************************/

/*
This program has been written to explain the use of the Joystick widget.
It reads a X11 Bitmap File and, with the joystick, the user can explore
the bitmap if its window is too small to display it in its entirety.
*/

#include <X11/Xlib.h>
#include <Xm/Xm.h>
#include <stdio.h> 
#include <stdlib.h>
#include "Joystick.h"
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <Xm/SelectioB.h>

XtAppContext context; 
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget form,toplevel,joystick,drawing_area,quit_but,read_but,promptdialog;

GC gc;

Pixmap bitmap;
unsigned int width,height;
int x_hot,y_hot,data_available;

Arg al[20]; 
int ac;

int mx,my; /* Middle of the drawing area. */
int px,py; /* The point in the bitmap which must coincide with (mx,my). */


/* To set the GC, we need only the background and foreground colors. */
void setup_gc()
{
int foreground,background;
XGCValues vals;
ac=0;
XtSetArg(al[ac],XmNforeground,&foreground);ac++;
XtSetArg(al[ac],XmNbackground,&background);ac++;
XtGetValues(drawing_area,al,ac);
vals.foreground = foreground;vals.background = background;
gc = XCreateGC(XtDisplay(drawing_area),
 RootWindowOfScreen(XtScreen(drawing_area)),GCForeground | GCBackground,&vals);
}


/* This procedure maps the bitmap on the drawing area.
   The joystick values are the bitmap's point which 
   coincides with the center of the drawing area. */
void exposureCB(w,client_data,call_data)
Widget w;
int client_data;
XtPointer call_data;
{
Dimension width_draw,height_draw;
int src_x,src_y,dest_x,dest_y,align,clear_win;
unsigned int width_area,height_area;
ac=0;
XtSetArg(al[ac],XmNwidth,&width_draw);ac++;
XtSetArg(al[ac],XmNheight,&height_draw);ac++;
XtGetValues(drawing_area,al,ac);
mx = (int)(width_draw)/2;
my = (int)(height_draw)/2;
clear_win = 0;
align = mx - px;
if (align >= 0) {src_x = 0;dest_x = align;clear_win = 1;}
else {src_x = -align;dest_x = 0;}
align = width - src_x;
if (align > width_draw) width_area = (unsigned int)(width_draw - dest_x);
else {width_area = (unsigned int)(width - src_x);clear_win = 1;}
align = my - py;
if (align >= 0) {src_y = 0;dest_y = align;clear_win = 1;}
else {src_y = -align;dest_y = 0;}
align = height - src_y;
if (align > height_draw) height_area = (unsigned int)(height_draw - dest_y);
else {height_area = (unsigned int)(height - src_y);clear_win = 1;}
if (clear_win) XClearWindow(XtDisplay(drawing_area),XtWindow(drawing_area));
XCopyPlane(XtDisplay(w),bitmap,XtWindow(drawing_area),gc,
          src_x,src_y,width_area,height_area,dest_x,dest_y,1);
}


/* This procedure returns the two values provided by the joystick. */
void joystick_callback(w,client_data,call_data)
Widget w;
XtPointer client_data;
XrmJoystickCallbackStruct *call_data;
{
ac=0;
XtSetArg(al[ac],XtNxValue,&px);ac++;
XtSetArg(al[ac],XtNyValue,&py);ac++;
XtGetValues(w,al,ac);
if (data_available) exposureCB(drawing_area,0,NULL);
}


void quit_but_callback(w,client_data,call_data)
Widget w;
int client_data;
XmAnyCallbackStruct *call_data;
{
exit(0);
}


/* This callback puts a dialog to get the X11 Bitmap File Name to display. */
void read_but_callback(w,client_data,call_data)
Widget w;
int client_data;
XmAnyCallbackStruct *call_data;
{
XtManageChild(promptdialog);
XtPopup(XtParent(promptdialog),XtGrabNone);
}


/* This procedure reads the X11 Bitmap File and sets the 
   two ranges of values for the joystick and exposes the drawing area. */
void prompt_callback(w,client_data,cbs)
Widget w;
XtPointer client_data;
XmSelectionBoxCallbackStruct *cbs;
{
int status;
char *value_str;
if (!XmStringGetLtoR(cbs->value,char_set,&value_str)) return;
XtFree(cbs->value);
if ((status = XReadBitmapFile(XtDisplay(drawing_area),XtWindow(drawing_area),
                 value_str,&width,&height,&bitmap,&x_hot,&y_hot))
    != BitmapSuccess) return;
data_available = 1;
px = width/2;
py = height/2;
ac=0;
XtSetArg(al[ac],XtNxMinimum,1);ac++;
XtSetArg(al[ac],XtNxMaximum,width);ac++;
XtSetArg(al[ac],XtNxValue,px);ac++;
XtSetArg(al[ac],XtNxProcessingDirection,XrmXMAX_ON_RIGHT);ac++;
XtSetArg(al[ac],XtNyMinimum,1);ac++;
XtSetArg(al[ac],XtNyMaximum,height);ac++;
XtSetArg(al[ac],XtNyValue,py);ac++;
XtSetArg(al[ac],XtNyProcessingDirection,XrmYMAX_ON_BOTTOM);ac++;
XtSetValues(joystick,al,ac);
XtAddCallback(drawing_area,XmNexposeCallback,exposureCB,NULL);
exposureCB(drawing_area,0,NULL);
}


void main(argc,argv)
int argc; 
char *argv[];
{
data_available = px = py = 0;

toplevel = XtAppInitialize(&context,"xreadbitmap",NULL,0,
                           &argc,argv,NULL,NULL,0);
form = (Widget)XmCreateForm(toplevel,"form",al,0);
XtManageChild(form);

ac=0;
XtSetArg(al[ac],XmNtopAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNrightAttachment,XmATTACH_POSITION);ac++;
XtSetArg(al[ac],XmNrightPosition,30);ac++;
XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_POSITION);ac++;
XtSetArg(al[ac],XmNbottomPosition,60);ac++;
joystick = XtCreateManagedWidget("joystick",XrmjoystickWidgetClass,form,al,ac);
XtAddCallback(joystick,XtNselect,joystick_callback,0);

ac=0;
XtSetArg(al[ac],XmNtopAttachment,XmATTACH_WIDGET);ac++;
XtSetArg(al[ac],XmNtopWidget,joystick);ac++;
XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_POSITION);ac++;
XtSetArg(al[ac],XmNbottomPosition,80);ac++;
XtSetArg(al[ac],XmNrightAttachment,XmATTACH_POSITION);ac++;
XtSetArg(al[ac],XmNrightPosition,30);ac++;
XtSetArg(al[ac],XmNlabelString,XmStringCreate("Read",char_set));ac++;
read_but = XmCreatePushButton(form,"read_but",al,ac);
XtManageChild(read_but);
XtAddCallback(read_but,XmNactivateCallback,read_but_callback,NULL);

ac=0;
XtSetArg(al[ac],XmNtopAttachment,XmATTACH_WIDGET);ac++;
XtSetArg(al[ac],XmNtopWidget,read_but);ac++;
XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNrightAttachment,XmATTACH_POSITION);ac++;
XtSetArg(al[ac],XmNrightPosition,30);ac++;
XtSetArg(al[ac],XmNlabelString,XmStringCreate("Quit",char_set));ac++;
quit_but = XmCreatePushButton(form,"quit_but",al,ac);
XtManageChild(quit_but);
XtAddCallback(quit_but,XmNactivateCallback,quit_but_callback,NULL);

ac=0;
XtSetArg(al[ac],XmNtopAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNleftAttachment,XmATTACH_WIDGET);ac++;
XtSetArg(al[ac],XmNleftWidget,joystick);ac++;
XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_FORM);ac++;
XtSetArg(al[ac],XmNrightAttachment,XmATTACH_FORM);ac++;
drawing_area = XmCreateDrawingArea(form,"drawing_area",al,ac);
XtManageChild(drawing_area);


/* Make the dialog to take the name of the bitmap file */
promptdialog = XmCreatePromptDialog(toplevel,"prompt",NULL,0);
XtUnmanageChild(XmSelectionBoxGetChild(promptdialog,XmDIALOG_HELP_BUTTON));
XtVaSetValues(promptdialog,
  XmNselectionLabelString,XmStringCreateLtoR("X11 Bitmap File Name:",char_set),
  XmNautoUnmanage,True,NULL);
XtAddCallback(promptdialog,XmNokCallback,prompt_callback,NULL);


setup_gc();

XtRealizeWidget(toplevel);
XtAppMainLoop(context);
}





